/* Copyright (c) 1996-2012 Clickteam
 *
 * This source code is part of the HTML5 exporter for Clickteam Multimedia Fusion 2.
 *
 * Permission is hereby granted to any person obtaining a legal copy
 * of Clickteam Multimedia Fusion 2 to use or modify this source code for
 * debugging, optimizing, or customizing applications created with
 * Clickteam Multimedia Fusion 2.
 * Any other use of this source code is prohibited.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */
 

CRunColorfx.CND_DUMMY = 0;
CRunColorfx.CND_DUMMY2 = 1;
CRunColorfx.CND_LAST = 2;

CRunColorfx.ACT_SETUNIFORM     = 0;
CRunColorfx.ACT_SETSPRNUM      = 1;
CRunColorfx.ACT_SETGSPEED       = 2;
 

CRunColorfx.ACT_LAST           = 3;


CRunColorfx.EXP_DUMMY = 0;
CRunColorfx.EXP_DUMMY2 = 1;

 
CRunColorfx.EXP_LAST = 2;

/*The MIT License (MIT)

Copyright (c) 2015 Lowww Ltd

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

var ShaderToy = function () {

    if (window.WebGLRenderingContext) {

       this.init();

    } else {

        console.error('You need a WebGL browser: Try get.webgl.org');

    }

};

ShaderToy.prototype = {
    constructor: ShaderToy.prototype,
    version: '0.1.0',
    mouse: new Float32Array(2), // 2 
    textures: [],
    width: 640,  // 320 240
    height: 480,
    _ready: false,
    _pause: false,
    _vertexShader: [
		 
        'attribute vec2 position;',
        'void main() {',
        '   gl_Position = vec4( position, 0.0, 1.0 );',
        '}'],
    _fragmentShader: [
	 
        'precision lowp float;',
		'uniform float iTime;',
		'uniform float seconds;',
	    'uniform vec3 iMouse;',
		'uniform vec2 iResolution;',
		'uniform sampler2D iChannel0;',
        'uniform sampler2D iChannel1;',
        'uniform sampler2D iChannel2;',
        'uniform sampler2D iChannel3;',
		// 'void mainImage( out vec4 fragColor, in vec2 fragCoord ) { fragColor.rgb = vec3( 0.5 + 0.5 * cos( iTime ) ); }'],
		 'void mainImage( out vec4 fragColor, in vec2 fragCoord ) { fragColor = texture2D(iChannel0, gl_FragCoord.xy); }',
		
       'void main() {',
       '   vec4 color = 4.*texture2D(iChannel0, gl_FragCoord.xy);',
       '   mainImage( color, gl_FragCoord.xy );',
       '   gl_FragColor = color;',
       '}'],

    _loadExternalFile: function (url) {

        return new Promise(function (resolve, reject) {

            var req = new XMLHttpRequest();
            req.open('GET', url);

            req.onload = function () {

                if (req.status === 200) {

                    resolve(req.response);

                } else {

                    reject(Error(req.statusText));

                }

            };

            req.onerror = function () {

                reject(Error('Network Error'));

            };

            req.send();

        });

    },

    _createProgram: function (vertexShader, fragmentShader) {
        var vs = this._createShader(vertexShader, this.gl.VERTEX_SHADER);
        var fs = this._createShader(fragmentShader, this.gl.FRAGMENT_SHADER);
        var program = this.gl.createProgram();
        this.gl.attachShader(program, vs);
        this.gl.attachShader(program, fs);
        this.gl.linkProgram(program);
        return program;
    },
    _createShader: function (str, type) {
        var shader = this.gl.createShader(type);
        this.gl.shaderSource(shader, str);
        this.gl.compileShader(shader);
		 
		
        var compiled = this.gl.getShaderParameter(shader, this.gl.COMPILE_STATUS);
        if (!compiled) {
            var error = this.gl.getShaderInfoLog(shader);
            console.error('Error compiling shader', shader, error);
            this.gl.deleteShader(shader);
            return null;
        }
        return shader;
    },
    _createTexture: function (index) {
        this.gl.activeTexture(this.gl.TEXTURE0 + (index));
        var texture2D = this.gl.createTexture();
	    this.gl.bindTexture(this.gl.TEXTURE_2D, texture2D);
				
        this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.RGBA, 1, 1, 0, this.gl.RGBA, this.gl.UNSIGNED_BYTE, new Uint8Array([0, 255, 255, 255]));
		
		 
		
        return texture2D;
    },
	 
    _updateTexture: function (buf, index) {
		
		 
        if (this.textures[index]) {
            this.gl.deleteTexture(this.textures[index]);
            delete this.textures[index];
            this._updateTexture(buf, index);
        } else {
            this.textures[index] = this._createTexture(index);
            this.gl.activeTexture(this.gl.TEXTURE0 + (index));
            this.gl.bindTexture(this.gl.TEXTURE_2D, this.textures[index]);
            this.gl.pixelStorei(this.gl.UNPACK_FLIP_Y_WEBGL, false);// true
            this.gl.texImage2D(this.gl.TEXTURE_2D, 0, this.gl.RGBA, this.gl.RGBA, this.gl.UNSIGNED_BYTE, buf);
           
		   
		   
 		   if (this._isPowerOf2(buf.width) && this._isPowerOf2(buf.height)) {
                this.gl.generateMipmap(this.gl.TEXTURE_2D);
            } else {
               // this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S, this.gl.REPEAT);
               // this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T, this.gl.REPEAT);
			   // this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MAG_FILTER, this.gl.LINEAR);
               // this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MIN_FILTER, this.gl.LINEAR_MIPMAP_LINEAR);
			   
			   // non power of two ;
			this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MAG_FILTER, this.gl.LINEAR);
            this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_MIN_FILTER, this.gl.LINEAR);
			this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_S, this.gl.CLAMP_TO_EDGE);
            this.gl.texParameteri(this.gl.TEXTURE_2D, this.gl.TEXTURE_WRAP_T, this.gl.CLAMP_TO_EDGE);
				
                
            }
        }
			 		
    },

    _getUniform: function (name) {

        if (!this.program.uniforms) {

            this.program.uniforms = {};

        }

        if (!this.program.uniforms[name]) {

            this.program.uniforms[name] = this.gl.getUniformLocation(this.program, name);

        }

        return this.program.uniforms[name];

    },

    _getAttribute: function (name) {

        if (!this.program.atttributes) {

            this.program.atttributes = {};

        }

        if (!this.program.atttributes[name]) {

            this.program.atttributes[name] = this.gl.getAttribLocation(this.program, name);

        }

        return this.program.atttributes[name];

    },

    _setU1f: function (name, x) {

        this.gl.uniform1f(this._getUniform(name), x);

    },

    _setU2f: function (name, x, y) {

        this.gl.uniform2f(this._getUniform(name), x, y);

    },
	_setU3f: function (name, x, y,z) {

        this.gl.uniform3f(this._getUniform(name), x, y,z);

    },
	_setU4f: function (name, x, y, z, w) {

        this.gl.uniform4f(this._getUniform(name), x, y,z,w);

    },

    // _setU2fv: function (name, v2) {

        // this.gl.uniform2fv(this._getUniform(name), v2);

    // },
// vec2 
		
	_setU2fv: function (name, x,y) {

        this.gl.uniform2fv(this._getUniform(name), x,y);

    },	
	
// vec3 
	_setU3fv: function (name, x,y,z) {

        this.gl.uniform3fv(this._getUniform(name), x,y,z);

    },
// vec4	
	_setU4fv: function (name, x,y,z,w) {

        this.gl.uniform4fv(this._getUniform(name), x, y, z, w);

    },
// float
    _setU1i: function (name, x) {

        this.gl.uniform1i(this._getUniform(name), x);

    },
	
	
    _onBlur: function (e) {

        this._pause = true;

    },

    _onFocus: function (e) {

        this._pause = false;

    },

    _onMove: function (e) {

        this.mouse[0] = e.layerX;
        this.mouse[1] = e.layerY;

    },

    _isPowerOf2: function (value) {

        return (value & (value - 1)) === 0;

    },

    init: function () {

        var canvas = document.createElement("canvas");
	 
		 
		canvas.width = this.width;
		canvas.height = this.height;
         
		// document.body.appendChild(canvas);
			 
			 
	  
         // window.addEventListener('blur', this._onBlur.bind(this));
         // window.addEventListener('focus', this._onFocus.bind(this));
         // window.addEventListener('mousemove', this._onMove.bind(this));
		this.gl = canvas.getContext('webgl');
		
		    this.gl.clearColor(1, 0, 0, 1);
			this.gl.clear(this.gl.COLOR_BUFFER_BIT);
			this.gl.enable(this.gl.BLEND);
			this.gl.blendFunc(this.gl.ONE, this.gl.ONE_MINUS_SRC_ALPHA);
			this.gl.disable(this.gl.CULL_FACE);
			this.gl.disable(this.gl.STENCIL_TEST);
			this.gl.disable(this.gl.DITHER); 
        
		this.gl.getExtension('OES_texture_float');
        this.gl.getExtension('OES_standard_derivatives');
        this.gl.getExtension('OES_float_linear');
        this.gl.getExtension('OES_half_float_linear');
 

        if (!this.gl) {

            console.error('Couldn\'t start WebGL. Try get.webgl.org/troubleshooting');

        }

        var args = [
            '\n%cShader%cToy%c v: ' + this.version + '\n',
            'background: #1A1A1A; color: #00ffff; font-size: x-large;',
            'background: #00ffff; color: #1A1A1A; font-size: x-large;',
            'background: transparent; color: #000000; font-size: x-small;'];

        if (!window.shadertoycredits) {

            window.shadertoycredits = true;
            console.log.apply(console, args);

        }
		 
        // quad
        var quad = new Float32Array([-1.0, -1.0, 1.0, -1.0, -1.0, 1.0, -1.0, 1.0, 1.0, -1.0, 1.0, 1.0]);
		 	
        var buffer = this.gl.createBuffer();
        this.gl.bindBuffer(this.gl.ARRAY_BUFFER, buffer);
        this.gl.bufferData(this.gl.ARRAY_BUFFER, quad, this.gl.STATIC_DRAW);

        // program
        var program = this._createProgram(this._vertexShader.join('\n'), this._fragmentShader.join('\n'));
        this.useProgram(program);

    },

    loadFragment: function (url) {

        this._ready = false;

        this._loadExternalFile(url).then(function (data) {

            this.writeFragment(data);

        }.bind(this));

    },
	
	loadFragmentText: function (data) {

       // this._ready = false;

         this.writeFragment(data);

    },
	
	loadImage: function (url, index) {
        var self = this;
        var image = new Image();
        image.src = url;
        image.index = index ;
        image.onload = function () {
            self._updateTexture(image, this.index); // slot ichannel 0-3
        };
    },

    writeFragment: function (str) {

        this._ready = false;

        this._fragmentShader[9] = str; // 8
        var program = this._createProgram(this._vertexShader.join('\n'), this._fragmentShader.join('\n'));
        this.useProgram(program);

    },

    useProgram: function (program) {

        this.program = program;
        this.gl.useProgram(this.program);

        this.gl.enableVertexAttribArray(this._getAttribute('position'));
        this.gl.vertexAttribPointer(this._getAttribute('position'), 2, this.gl.FLOAT, false, 0, 0);

        this._setU2f('iResolution', this.width, this.height);

        this.startTime = Date.now();
        this._ready = true;

    },

    setSize: function (width, height) {

        this.width = width;
        this.height = height;

        this.gl.canvas.width = this.width;
        this.gl.canvas.height = this.height;

        this.gl.viewportWidth = this.width;
        this.gl.viewportHeight = this.height;
        this.gl.viewport(0, 0, this.width, this.height);
		
		

        if (this.program) {

            this._setU2f('iResolution', this.width, this.height);

        }

    },

    clear: function (r, g, b, a) {

        this.gl.clearColor(r || 0, g || 0, b || 0, a || 1);
        this.gl.clear(this.gl.COLOR_BUFFER_BIT | this.gl.DEPTH_BUFFER_BIT);

    },

    render: function () {

        if (this._ready !== true) return;
        if (this._pause) return;

        this._setU1f('iTime', (Date.now() - this.startTime) / 1000) ||
		this._setU1f('seconds', (Date.now() - this.startTime) / 1000);
       
       // this._setU2fv('iMouse',1,1,1);
						
        this._setU1i('iChannel0', 0);
        this._setU1i('iChannel1', 1);
        this._setU1i('iChannel2', 2);
        this._setU1i('iChannel3', 3);
		 

       // this.clear();
        this.gl.drawArrays(this.gl.TRIANGLES, 0, 6);

    }

};

var st;
 
var spnum=0;
var ndex = 0;
		 
 
function CRunColorfx()
{
	 
}
CRunColorfx.prototype = CServices.extend(new CRunControl(),
	{
		getNumberOfConditions: function ()
		{
			return CRunColorfx.CND_LAST;
		},

		createRunObject: function (file, cob, version)
		{
			this.ho.hoImgWidth = file.readAInt();
			this.ho.hoImgHeight = file.readAInt();
			
				var w = this.ho.hoImgWidth;
				var h = this.ho.hoImgHeight;
				
			st = new ShaderToy();
			 
			var fragment = [
			
			'vec3 multiply(in vec3 src, in vec3 dst)',
			'{',
			'return src * dst;',
			'}',
			'uniform float xs;',
			'uniform float red;',
			'uniform float green;',
			'uniform float blue;',
			
			'void mainImage( out vec4 fragColor, in vec2 fragCoord ) {  ',       
			'    float time = iTime * 0.3 + iMouse.x*0.01;              ',       
			'	                                                        ',       
			'     vec2 uv =  fragCoord.xy/iResolution.xy;               ',
			' 	    // uv = floor(256.*uv)/256.0;                        ',
			
			'       uv.y = 1.-uv.y; 									',
			
			'     vec4 tx =  texture2D(iChannel0,uv);               ',
		 
			'	//tx.r = tx.r*c1;                                     ',
			'	//tx.g = tx.g*c2;									   ',		
			'	//tx.b = tx.b*c3;									   ',	

			'   tx.r = texture2D(iChannel0,uv.xy+vec2(0.0,0.0)).r*red;     ',
			'   tx.g = texture2D(iChannel0,uv.xy+vec2(0.0,0.0)).g*green;   ',
			'   tx.b = texture2D(iChannel0,uv.xy-vec2(0.0,0.0)).b*blue;    ',
			
			
			      
			'	  fragColor = vec4(tx);                       ',       

			
			'}',                 
			].join( '\n' );
			 
			 var img = "resources/M0001.png";
				
			 
			 st.writeFragment(fragment);
										
			 st.loadImage(img,0);				 
						

			return true;
		},

		displayRunObject: function (context, xDraw, yDraw)
		{
			 
				var x = this.ho.hoX - this.rh.rhWindowX + this.ho.pLayer.x ;
                var y = this.ho.hoY - this.rh.rhWindowY + this.ho.pLayer.y ;	
			 
				//var x = this.ho.hoX  ;
				//	var y = this.ho.hoY  ;
				var w = this.ho.hoImgWidth;
				var h = this.ho.hoImgHeight;
				
				 
				var ccx = this.rh.rhApp.canvas.getContext("2d");
				
				    
				var ndxel = this.rh.rhApp.imageBank.images.length;
			
			
				// context.renderSimpleImage(st.gl.canvas,x,y,w,h,0,0);
				if(ndex<ndxel) {
				var imw = this.rh.rhApp.imageBank.images[ndex].width; // 500
				var imh = this.rh.rhApp.imageBank.images[ndex].height;// 300
				var imX = this.rh.rhApp.imageBank.images[ndex].mosaicX; // posX dans mosaic
				var imY = this.rh.rhApp.imageBank.images[ndex].mosaicY; // posY dans mosaic
				
				 }
				 else {
						window.alert("HTML5 Color Fx Extension : Image index out of range ("+ndxel+")");
						return;
				 }
				
				
				// posx, y dans canvas handle num ; 
				// var handle = this.rh.rhApp.frame.LOList.list[99].loOiHandle;
				
				
				// var px = this.rh.rhApp.frame.LOList.list[99].loX;
				// var py = this.rh.rhApp.frame.LOList.list[99].loY;
				
				st.setSize(this.rh.rhApp.imageBank.mosaics[1].width,this.rh.rhApp.imageBank.mosaics[1].height);	
				 
				 
				 
				 ccx.drawImage(st.gl.canvas,imX,imY,imw,imh,x,y,w,h);
				 
				
				// context.renderSimpleImage(st.gl.canvas,
							  // x+xDraw, y+yDraw,
							  // this.rh.rhApp.imageBank.images[3].width, this.rh.rhApp.imageBank.images[3].width,
                              // 0, 0);
							
				 		
							
				 
							
			 
		},

		handleRunObject: function ()
		{
			 st.render();
			  return 0;
		},

		condition: function (num, cnd)
		{
			switch (num)
			{
				 case CRunColorfx.CND_DUMMY:
                var parameter = cnd.getParamExpression(this.rh, 0);
                return (parameter == 0);
				
				case CRunColorfx.CND_DUMMY2:
                var string = cnd.getParamExpString(this.rh, 0);
                var fromAction = this.rh.rhEvtProg.rhCurParam0;
                if (string == fromAction)
                    return true;
                break;
				
			}
			return false;
		},

		cndPlaying: function ()
		{
			 
			return false;
		},

		actStartURL: function (act)
		{
			 
		},

		action: function (num, act)
		{
			switch (num)
			{
			
							
			case CRunColorfx.ACT_SETUNIFORM:
				st._setU1f(act.getParamExpString(this.rh, 0),act.getParamExpression(this.rh, 1));
				break;
			case CRunColorfx.ACT_SETSPRNUM:
				      ndex = act.getParamExpression(this.rh, 0);
				break;
			case CRunColorfx.ACT_SET_UNIFORM_VEC3:
				st._setU3f(act.getParamExpString(this.rh, 0),act.getParamExpression(this.rh, 1),act.getParamExpression(this.rh, 2),act.getParamExpression(this.rh, 3));
				break;
			case CRunColorfx.ACT_SET_UNIFORM_VEC4:
				st._setU4f(act.getParamExpString(this.rh, 0),act.getParamExpression(this.rh, 1),act.getParamExpression(this.rh, 2),act.getParamExpression(this.rh, 3),act.getParamExpression(this.rh, 4));
				break;			
		 

				
			}
		},

		expression: function (num)
		{
			switch (num)
			{
				
				case CRunColorfx.EXP_DUMMY:
                var param1 = this.ho.getExpParam();     // Get first parameter
                var param2 = this.ho.getExpParam();     // Get second parameter
                return param1 + param2;
            
            // Dummy expression : returns the length of a string
            case CRunColorfx.EXP_DUMMY2:
                var string = this.ho.getExpParam();     // Get the string parameter
                return string.length;
				
			}
			return 0;
		}
	});
